package com.huatek.busi.mapper;

import com.github.pagehelper.Page;
import com.huatek.busi.dto.LabReportBasisDataDto;
import com.huatek.busi.entity.BusiOfflineGisBrand;
import com.huatek.busi.vo.BusiSiteTreeVo;
import com.huatek.busi.where.LabReportBasisDataSelectVo;
import org.apache.ibatis.annotations.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface LabReportBasisDataMapper {

    List<LabReportBasisDataDto> list(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    Page<LabReportBasisDataDto> findExcelList(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    int delete(@Param("refId") String refId);

    int updateByRefIds(@Param("refIds") List<String> refIds);

    String findId(String equlittleId);

    String findLastFile(@Param("eqType") String eqType, @Param("key") String key);

    int deleteBlq(@Param("refId") String refId);

    int deleteDl(@Param("refId") String refId);

    int deleteKgg(@Param("refId") String refId);

    int deleteGen(@Param("refId") String refId);

    List<LabReportBasisDataDto> listBlq(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    List<LabReportBasisDataDto> listDl(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    List<LabReportBasisDataDto> listKgg(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    List<LabReportBasisDataDto> listGen(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    Page<LabReportBasisDataDto> findBlqExcelList(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    Page<LabReportBasisDataDto> findDlExcelList(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    Page<LabReportBasisDataDto> findKggExcelList(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    Page<LabReportBasisDataDto> findGenExcelList(LabReportBasisDataSelectVo labReportBasisDataSelectVo);

    int updateBlqByRefIds(@Param("refIds") List<String> refIds);

    int updateDlByRefIds(@Param("refIds") List<String> refIds);

    int updateKggByRefIds(@Param("refIds") List<String> refIds);

    int updateGenByRefIds(@Param("refIds") List<String> refIds);

    BusiSiteTreeVo findGisInfo(@Param("equipmentId") String equipmentId, @Param("jgId") String jgId);

    BusiSiteTreeVo findByqInfo(@Param("equipmentId") String equipmentId);

    String findEquTypeId(@Param("equipmentId") String equipmentId);

    BusiOfflineGisBrand findBusiOfflineGisBrand(@Param("equipmentId") String equipmentId);
}
